#include "StdAfx.h"
#include "DLBPCS.h"

const int SamplePoints[][2]={{1,0},{1,1},{0,1},{-1,1},{-1,0},{-1,-1},{0,-1},{1,-1}};//x,y

DLBPCS::DLBPCS()
{
	m_pAddrOff=NULL;
	m_pAddrOff2=NULL;
	m_pLUTSubuniform=NULL;
}

DLBPCS::~DLBPCS()
{
	Free();
}

void DLBPCS::Initial(int w,int h)
{
	Free();
	int size,i,p;
	size=w*h;
	this->m_iWidth0=w;
	this->m_iHeight0=h;
	//1. allocate memory
	m_pAddrOff=new int[8];
	m_pAddrOff2=new int[8];
	for (i=0;i<8;i++)
	{
		m_pAddrOff[i]=SamplePoints[i][0]+SamplePoints[i][1]*w;
		m_pAddrOff2[i]=m_pAddrOff[i]*2;
	}

	int i,num;
	BYTE bi;
	//2. Calculate the number of bit "1" in a code (0~255)
	for (i=0;i<256;i++)
	{
		bi=BYTE(i);
		num = 0;
		while(bi)
		{
			bi&=(bi - 1);
			num++;
		}
		m_pBitOneNum[i]=num;
	}
	//3. calculate the lookup table for sub-uniform
	m_pLUTSubuniform=new CSubuniform[256];
	int iCode,n;
	int ShiftOne[8];
	ShiftOne[0]=1;
	ShiftOne[1]=2;
	ShiftOne[2]=4;
	ShiftOne[3]=8;
	ShiftOne[4]=16;
	ShiftOne[5]=32;
	ShiftOne[6]=64;
	ShiftOne[7]=128;
	for (iCode=0;iCode<256;iCode++)
	{
		if(m_pTransitNum[iCode]>2)
		{//non-uniform pattern
			m_pLUTSubuniform[iCode].m_BitOneNum=9;//P+1
			m_pLUTSubuniform[iCode].m_BitOneIdx=-1;//invalid flag
		}else if(m_pTransitNum[iCode]==2)
		{//2
			m_pLUTSubuniform[iCode].m_BitOneNum=m_pBitOneNum[iCode];//1,2,...,7
			for (n=0;n<8;n++)
			{
				if(((iCode&ShiftOne[n])!=0)&&((iCode&ShiftOne[(n+7)%8])==0))
				{
					m_pLUTSubuniform[iCode].m_BitOneIdx=n;//0,1,...,7
					break;
				}
			}
		}else
		{//0
			m_pLUTSubuniform[iCode].m_BitOneNum=m_pBitOneNum[iCode];//0 or 8
			m_pLUTSubuniform[iCode].m_BitOneIdx=-1;//invalid flag
		}
	}
}
void DLBPCS::Free()
{
	if(m_pAddrOff)
		delete []m_pAddrOff;
	m_pAddrOff=NULL;
	if(m_pAddrOff2)
		delete []m_pAddrOff2;
	m_pAddrOff2=NULL;
	if(m_pLUTSubuniform)
		delete []m_pLUTSubuniform;
	m_pLUTSubuniform=NULL;
}
//******Input parameters******
	//ppI: pointers to the different scale images of an image 
	//pScale: the optimal scale for each pixel
	//ppI and pScale should be pre-calculated for an image by the method 
	//described in the section 3.2 of the paper entitled "Rotation and Scale Invariant Local Binary Pattern Based on
	//High Order Directional Derivatives for Texture Classification."
	//w,h: the image size
//******Output parameters******
	//pSubuniform1: sub-uniforms for 1st order
	//pSubuniform2: sub-uniforms for 2nd order
void DLBPCS::Code(float** ppI,int *pScale,int w,int h,CSubuniform *pSubuniform1,CSubuniform *pSubuniform2)
{//do not process the borders of the images
	int w2,h2,i;
	int w1,h1;
	int x,y;
	int code1,code2;
	int iAddr=(w+1)*2;
	int scale,idx,rx,ry,rm;
	int halfw=w/2,halfh=h/2;
	int *pAddrOff;
	float *pI;
	w1=w-1;
	h1=h-1;
	w2=w-2;
	h2=h-2;

	for(y=2;y<h2;y++)
	{
		if(y<halfh)
			ry=y;
		else
			ry=h1-y;
		for(x=2;x<w2;x++,iAddr++)
		{
			if(x<halfw)
				rx=x;
			else
				rx=w1-x;
			if(ry<rx)
				rm=ry;
			else
				rm=rx;
			rm/=2;
			if(rm<pScale[iAddr])
				scale=rm;
			else
				scale=pScale[iAddr];
			//Calculate the original 1st order LBP code and 2nd order LBP code
			idx=scale-1;
			pAddrOff=m_ppAddrOff[idx];
			pI=ppI[idx]+iAddr;
			code1=0;
			code2=0;
			for (i=0;i<8;i++)
			{
				if(pI[pAddrOff[i]]>=pI[0])
					code1=code1+(1<<i);//1st derivative
				if(pI[0]+pI[pAddrOff[i]*2]>=pI[pAddrOff[i]]*2.0f)
					code2=code2+(1<<i);//2nd derivative
				//if(pI[m_pAddrOff[i]]>=pI[0])
				//	code1=code1+(1<<i);//1st derivative
				//if(pI[0]+pI[m_pAddrOff2[i]]>=pI[m_pAddrOff[i]]*2.0f)
				//	code2=code2+(1<<i);//2nd derivative
			}
			//Mapping the original 1st order LBP code and 2nd order LBP code into sub-uniforms
			pSubuniform1[iAddr]=m_pLUTSubuniform[code1];
			pSubuniform2[iAddr]=m_pLUTSubuniform[code2];
		}
		iAddr+=4;
	}
}
//Calculate the histogram of a region specified by the coordinates (x0,y0,x1,y1) for single order DLBPCS codes
void CLbp::Hist(CSubuniform *pSubuniform,int w,int h,int x0,int y0,int x1,int y1,int *pHist)
{
	int x,y,n;
	for(n=0;n<59;n++)
		pHist[n]=0;
	CSubuniform *pSubX;
	CSubuniform *pSubY=pSubuniform+y0*w+x0;
	int *pM;
	int *ppM[8];
	ppM[1]=pHist;
	ppM[2]=pHist+8;
	ppM[3]=pHist+16;
	ppM[4]=pHist+24;
	ppM[5]=pHist+32;
	ppM[6]=pHist+40;
	ppM[7]=pHist+48;
	for(y=y0;y<y1;y++)
	{
		pSubX=pSubY;
		for(x=x0;x<x1;x++)
		{
#ifdef _DEBUG
			if(pSubX->m_BitOneNum>9||pSubX->m_BitOneNum<0)
				return;
			if((pSubX->m_BitOneNum<=7&&pSubX->m_BitOneNum>=1)&&
				(pSubX->m_BitOneIdx>7||pSubX->m_BitOneIdx<0))//0,...,7
				return;
#endif
			switch(pSubX->m_BitOneNum)
			{
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				ppM[pSubX->m_BitOneNum][pSubX->m_BitOneIdx]++;//7*8
				break;
			case 8://0xFF
				pHist[56]++;
				break;
			case 0://0x00
				pHist[57]++;
				break;
			case 9://non-uniform
				pHist[58]++;
				break;
			}
			pSubX++;
		}
		pSubY+=w;
	}
	//circular shift 
	int i,iMax,idxMax;
	int tmp[8];
	pM=pHist;
	for(n=0;n<7;n++)
	{
		tmp[0]=pM[0];//copying data
		iMax=pM[0];
		idxMax=0;
		//find dominant dir
		for(i=1;i<8;i++)
		{
			if(iMax<pM[i])
			{
				iMax=pM[i];
				idxMax=i;
			}
			tmp[i]=pM[i];//copying data
		}
		//circular shift
		for(i=0;i<8;i++)
		{
			pM[i]=tmp[(i+idxMax)%8];
		}
		pM+=8;
	}
}
//Calculate the histogram of a region specified by the coordinates (x0,y0,x1,y1) for the 1st and 2nd order DLBPCS codes
inline void CLbp::Hist(CSubuniform *pSub1,CSubuniform *pSub2,int w,int h,int x0,int y0,int x1,int y1,int *pHist1)
{
	int *pHist2=pHist1+59;
	Hist(pSub1,w,h,x0,y0,x1,y1,pHist1);
	Hist(pSub2,w,h,x0,y0,x1,y1,pHist2);
}