#ifndef DLBPCS_Feiniu_header
#define DLBPCS_Feiniu_header
/* 
* Feiniu Yuan. Rotation and Scale Invariant Local Binary Pattern Based on 
* High Order Directional Derivatives for Texture Classification. 
* Digital Signal Processing, Volume 26, Issue 3, 2014, pp. 142-152.
* 
* Copyright (C) 2015 Feiniu Yuan, all rights reserved.
* Prof. Feiniu Yuan
* http://sit.jxufe.cn/grbk/yfn/
* E-mail: yfn@ustc.edu, 30813288@qq.com
* 
*/
class  CSubuniform
{
public:
	char m_BitOneNum;//Uniform: 0,1, ..., 8,  Non-uniform: 9
	char m_BitOneIdx;
};

class DLBPCS
{
public:
	DLBPCS();
	~DLBPCS();
	void Initial(int w,int h);
	void Free();
	//******Input parameters******
	//ppI: pointers to the different scale images of an image 
	//pScale: the optimal scale for each pixel
	//ppI and pScale should be pre-calculated for an image by the method 
	//described in the section 3.2 of the paper entitled "Rotation and Scale Invariant Local Binary Pattern Based on
	//High Order Directional Derivatives for Texture Classification."
	//w,h: the image size
	//******Output parameters******
	//pSubuniform1: sub-uniforms for 1st order
	//pSubuniform2: sub-uniforms for 2nd order
	void Code(float** ppI,int *pScale,int w,int h,CSubuniform *pSubuniform1,CSubuniform *pSubuniform2);
	//Calculate the histogram of a region specified by the coordinates (x0,y0,x1,y1) for single order DLBPCS codes
	void Hist(CSubuniform *pSubuniform,int w,int h,int x0,int y0,int x1,int y1,int *pHist);
	//Calculate the histogram of a region specified by the coordinates (x0,y0,x1,y1) for the 1st and 2nd order DLBPCS codes
	void Hist(CSubuniform *pSubuniform1,CSubuniform *pSubuniform2,int w,int h,int x0,int y0,int x1,int y1,int *pHist12);
public:
	int				m_iWidth0,m_iHeight0;//the image size
	int*			m_pAddrOff;//address offsets with radius=1 for 1st order directional derivatives of neighbors
	int*			m_pAddrOff2;//address offsets with radius=1 for 2nd order directional derivatives of neighbors
	BYTE			m_pBitOneNum[256];
	CSubuniform	*	m_pLUTSubuniform;//sub-uniform lookup table
};
#endif